@@ -65,9 +65,9 @@ function init(aEvent)
     // Pref is unset
   }
 
-  // Include the build ID if this is a "pre" (i.e. non-release) build
+  // Include the build ID if this is an "a#" (nightly or aurora) build
   let version = Services.appinfo.version;
-  if (version.indexOf("pre") != -1) {
+  if (/a\d+$/.test(version)) {
     let buildID = Services.appinfo.appBuildID;
     let buildDate = buildID.slice(0,4) + "-" + buildID.slice(4,6) + "-" + buildID.slice(6,8);
     document.getElementById("version").value += " (" + buildDate + ")";
@@ -88,6 +88,10 @@ function init(aEvent)
   gAppUpdater = new appUpdater();
 #endif
 
+  let defaults = Services.prefs.getDefaultBranch("");
+  let channelLabel = document.getElementById("currentChannel");
+  channelLabel.value = defaults.getCharPref("app.update.channel");
+
 #ifdef XP_MACOSX
   // it may not be sized at this point, and we need its width to calculate its position
   window.sizeToContent();
@@ -357,7 +361,6 @@ appUpdater.prototype =
       // notified with the normal app update user interface so this is safe.
       gAppUpdater.isChecking = false;
       gAppUpdater.selectPanel("noUpdatesFound");
-      return;
     },
 
     /**
