@@ -582,11 +582,13 @@ var gCookiesWindow = {
     if (item && seln.count == 1 && item.container && item.open)
       selectedCookieCount += 2;
 
-    var stringKey = selectedCookieCount == 1 ? "removeCookie" : "removeCookies";
-    document.getElementById("removeCookie").label = this._bundle.getString(stringKey);
+    var removeCookie = document.getElementById("removeCookie");
+    var removeCookies = document.getElementById("removeCookies");
+    removeCookie.parentNode.selectedPanel =
+      selectedCookieCount == 1 ? removeCookie : removeCookies;
 
     document.getElementById("removeAllCookies").disabled = this._view._filtered;
-    document.getElementById("removeCookie").disabled = !(seln.count > 0);
+    removeCookie.disabled = removeCookies.disabled = !(seln.count > 0);
   },
 
   deleteCookie: function () {
@@ -760,6 +762,19 @@ var gCookiesWindow = {
         this._view._filterSet.reverse();
     }
 
+    // Adjust the Sort Indicator
+    var domainCol = document.getElementById("domainCol");
+    var nameCol = document.getElementById("nameCol");
+    var sortOrderString = ascending ? "ascending" : "descending";
+    if (aProperty == "rawHost") {
+      domainCol.setAttribute("sortDirection", sortOrderString);
+      nameCol.removeAttribute("sortDirection");
+    }
+    else {
+      nameCol.setAttribute("sortDirection", sortOrderString);
+      domainCol.removeAttribute("sortDirection");
+    }
+
     this._view._invalidateCache(0);
     this._view.selection.clearSelection();
     this._view.selection.select(0);
@@ -895,5 +910,10 @@ var gCookiesWindow = {
     var filter = document.getElementById("filter");
     filter.focus();
     filter.select();
+  },
+
+  onWindowKeyPress: function (aEvent) {
+    if (aEvent.keyCode == KeyEvent.DOM_VK_ESCAPE)
+      window.close();
   }
 };
