@@ -67,6 +67,7 @@
 #include "nsIDocumentEncoder.h"
 #include "nsIAnimationFrameListener.h"
 #include "nsEventStates.h"
+#include "nsIStructuredCloneContainer.h"
 
 class nsIContent;
 class nsPresContext;
@@ -79,7 +80,7 @@ class nsCSSStyleSheet;
 class nsIViewManager;
 class nsIDOMEvent;
 class nsIDOMEventTarget;
-class nsIDeviceContext;
+class nsDeviceContext;
 class nsIParser;
 class nsIDOMNode;
 class nsIDOMElement;
@@ -123,8 +124,8 @@ class Element;
 
 
 #define NS_IDOCUMENT_IID      \
-{ 0xc38a7935, 0xc854, 0x4df7, \
-  { 0x8f, 0xd4, 0xa2, 0x6f, 0x0d, 0x27, 0x9f, 0x31 } }
+{ 0x26ef6218, 0xcd5e, 0x4953,  \
+ { 0xbb, 0x57, 0xb8, 0x50, 0x29, 0xa1, 0xae, 0x40 } }
 
 // Flag for AddStyleSheet().
 #define NS_STYLESHEET_FROM_CATALOG                (1 << 0)
@@ -166,7 +167,7 @@ public:
       mIsBeingUsedAsImage(PR_FALSE),
       mPartID(0)
   {
-    mParentPtrBits |= PARENT_BIT_INDOCUMENT;
+    SetInDocument();
   }
 #endif
   
@@ -689,7 +690,7 @@ public:
   /**
    * Return the window containing the document (the outer window).
    */
-  nsPIDOMWindow *GetWindow()
+  nsPIDOMWindow *GetWindow() const
   {
     return mWindow ? mWindow->GetOuterWindow() : GetWindowInternal();
   }
@@ -707,7 +708,7 @@ public:
   /**
    * Return the outer window ID.
    */
-  PRUint64 OuterWindowID()
+  PRUint64 OuterWindowID() const
   {
     nsPIDOMWindow *window = GetWindow();
     return window ? window->WindowID() : 0;
@@ -757,11 +758,9 @@ public:
   virtual void SetReadyStateInternal(ReadyState rs) = 0;
   virtual ReadyState GetReadyStateEnum() = 0;
 
-  // notify that one or two content nodes changed state
-  // either may be nsnull, but not both
-  virtual void ContentStatesChanged(nsIContent* aContent1,
-                                    nsIContent* aContent2,
-                                    nsEventStates aStateMask) = 0;
+  // notify that a content node changed state
+  virtual void ContentStateChanged(nsIContent* aContent,
+                                   nsEventStates aStateMask) = 0;
 
   // Notify that a document state has changed.
   // This should only be called by callers whose state is also reflected in the
@@ -1424,12 +1423,13 @@ public:
   };
 
   /**
-   * Set the document's pending state object (as serialized to JSON).
+   * Set the document's pending state object (as serialized using structured
+   * clone).
    */
-  void SetCurrentStateObject(nsAString &obj)
+  void SetStateObject(nsIStructuredCloneContainer *scContainer)
   {
-    mCurrentStateObject.Assign(obj);
-    mCurrentStateObjectCached = nsnull;
+    mStateObjectContainer = scContainer;
+    mStateObjectCached = nsnull;
   }
 
   /**
@@ -1517,6 +1517,8 @@ public:
   // state is unlocked/false.
   virtual nsresult SetImageLockingState(PRBool aLocked) = 0;
 
+  virtual nsresult GetStateObject(nsIVariant** aResult) = 0;
+
 protected:
   ~nsIDocument()
   {
@@ -1529,7 +1531,7 @@ protected:
   nsPropertyTable* GetExtraPropertyTable(PRUint16 aCategory);
 
   // Never ever call this. Only call GetWindow!
-  virtual nsPIDOMWindow *GetWindowInternal() = 0;
+  virtual nsPIDOMWindow *GetWindowInternal() const = 0;
 
   // Never ever call this. Only call GetInnerWindow!
   virtual nsPIDOMWindow *GetInnerWindowInternal() = 0;
@@ -1724,8 +1726,6 @@ protected:
    */
   PRUint32 mExternalScriptsBeingEvaluated;
 
-  nsString mCurrentStateObject;
-
   // Weak reference to mScriptGlobalObject QI:d to nsPIDOMWindow,
   // updated on every set of mSecriptGlobalObject.
   nsPIDOMWindow *mWindow;
@@ -1741,7 +1741,8 @@ protected:
   // Our base target.
   nsString mBaseTarget;
 
-  nsCOMPtr<nsIVariant> mCurrentStateObjectCached;
+  nsCOMPtr<nsIStructuredCloneContainer> mStateObjectContainer;
+  nsCOMPtr<nsIVariant> mStateObjectCached;
 };
 
 NS_DEFINE_STATIC_IID_ACCESSOR(nsIDocument, NS_IDOCUMENT_IID)
