@@ -41,7 +41,7 @@
  * ***** END LICENSE BLOCK ***** */
 
 
-START("9.2.1.5 XMLList [[HasProperty]]");
+START("9.1.1.10 - XML [[ResolveValue]]");
 
 var actual = 'Test not implemented';
 var expect = 'PASS';
@@ -49,5 +49,3 @@ var expect = 'PASS';
 TEST(1, expect, actual);
 
 END();
-
-
