@@ -16,10 +16,11 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2006
+ * Portions created by the Initial Developer are Copyright (C) 2005
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s): Phil Schwartau <pschwartau@meer.net>
+ *                 Bob Clary <bob@bclary.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,37 +37,47 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 352742;
-var summary = 'Array filter on {valueOf: Function}';
+var BUGNUMBER = 99120;
+var summary = 'sort() should not be O(N^2) on sorted data';
 var actual = '';
 var expect = '';
 
+printBugNumber(BUGNUMBER);
+printStatus (summary);
 
-//-----------------------------------------------------------------------------
-test();
-//-----------------------------------------------------------------------------
+var data = {X:[], Y:[]};
+for (var size = 5000; size <= 15000; size += 1000)
+{ 
+  data.X.push(size);
+  data.Y.push(testSort(size));
+  gc();
+}
 
-function test()
+var order = BigO(data);
+
+var msg = '';
+for (var p = 0; p < data.X.length; p++)
 {
-  enterFunc ('test');
-  printBugNumber(BUGNUMBER);
-  printStatus (summary);
+  msg += '(' + data.X[p] + ', ' + data.Y[p] + '); ';
+}
+printStatus(msg);
+printStatus('Order: ' + order);
+reportCompare(true, order < 2 , 'BigO ' + order + ' < 2');
 
-  print('If the test harness fails, this test fails.');
-  expect = 4; 
-  z = {valueOf: Function};
-  actual = 2;
-  try {
-    [11].filter(z);
-  }
-  catch(e)
+function testSort(size)
+{
+  var arry = new Array(size);
+  for (var i = 0; i < size; i++)
   {
-    actual = 3;
-    print(e);
+    arry[i] = i;
   }
-  actual = 4;
+  var start = new Date();
+  arry.sort(compareFn);
+  var stop = new Date();
+  return stop - start;
+}
 
-  reportCompare(expect, actual, summary);
-
-  exitFunc ('test');
+function compareFn(a, b)
+{
+  return a - b;
 }
