@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2005
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Brendan Eich <brendan@mozilla.org>
+ * Contributor(s): Igor Bukanov  <igor@mir2.org>
  *                 Bob Clary <bob@bclary.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
@@ -37,43 +37,44 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 230216;
-var summary = 'check for numerical overflow in regexps in back reference and bounds for {} quantifier';
-var actual = '';
-var expect = '';
-var status = '';
+var BUGNUMBER = 280769;
+var summary = 'Do not overflow 64K length of char sequence in RegExp []';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
 printBugNumber(BUGNUMBER);
 printStatus (summary);
 
-status = inSection(1) + ' /((\3|b)\2(a)x)+/.exec("aaxabxbaxbbx") ';
 
-actual = 'undefined'; 
-expect = ['ax', 'ax', '', 'a'] + '';
+var N = 20 * 1000; // It should be that N*6  > 64K and N < 32K
 
-try
-{ 
-  actual = /((\3|b)\2(a)x)+/.exec("aaxabxbaxbbx") + '';
-}
-catch(e)
+var prefixes = ["000", "00", "0"];
+
+function to_4_hex(i)
 {
-  status += ' Error: ' + e;
+  var printed = i.toString(16);
+  if (printed.length < 4) {
+    printed= prefixes[printed.length - 1]+printed;
+  }
+  return printed;
+
 }
 
-reportCompare(expect, actual, status);
-
-status = inSection(2) + ' eval(\'/((\3|b)\2(a)x)+/.exec("aaxabxbaxbbx")\' ';
-
-actual = 'undefined'; 
-expect = ['ax', 'ax', '', 'a'] + '';
-
-try
-{ 
-  actual = eval('/((\\3|b)\\2(a)x)+/.exec("aaxabxbaxbbx")') + '';
-}
-catch(e)
-{
-  status += ' Error: ' + e;
+var a = new Array(N);
+for (var i = 0; i != N; ++i) {
+  a[i] = to_4_hex(2*i);
 }
 
-reportCompare(expect, actual, status);
+var str = '[\\u'+a.join('\\u')+']';
+// str is [\u0000\u0002\u0004...\u<printed value of 2N>]
+
+var re = new RegExp(str);
+
+var string_to_match = String.fromCharCode(2 * (N - 1));
+
+var value = re.exec(string_to_match);
+
+var expect =  string_to_match;
+var actual = value ? value[0] : value;
+
+reportCompare(expect, actual, summary);
