@@ -3,26 +3,29 @@
  * http://creativecommons.org/licenses/publicdomain/
  */
 
-test();
+function p() { }
 
 function test()
 {
   var counter = 0;
-  function f(x,y) {
+
+  function f(x) {
       try
       { 
         throw 42;
       }
-      catch(e2)
+      catch(e)
       { 
-        foo(function(){ return x; }| "9.2" && 5 || counter && e);
-        ++counter;
+        assertEq(counter, 0);
+        p(function(){x;});
+        counter = 1;
       }
   }
 
-  f(2, 1);
+  f(2);
+  assertEq(counter, 1);
 }
 
-function foo(bar) { return ""+bar; }
+test();
 
 reportCompare(0, 0, "ok");
