@@ -4,9 +4,9 @@ function dumpn(s) {
 
 const NS_APP_USER_PROFILE_50_DIR = "ProfD";
 const NS_APP_USER_PROFILE_LOCAL_50_DIR = "ProfLD";
-const Ci = Components.interfaces;
-const Cc = Components.classes;
-const Cr = Components.results;
+
+// This loads Ci, Cc, Cr and Cu.
+do_load_httpd_js();
 
 do_get_profile();
 
@@ -188,7 +188,7 @@ checkUrls: function(urls, expected, cb)
     } else {
       cb();
     }
-  }
+  };
   doLookup();
 },
 
