@@ -240,7 +240,7 @@ nsresult nsCocoaWindow::Create(nsIWidget *aParent,
                                nsNativeWidget aNativeParent,
                                const nsIntRect &aRect,
                                EVENT_CALLBACK aHandleEventFunction,
-                               nsIDeviceContext *aContext,
+                               nsDeviceContext *aContext,
                                nsIAppShell *aAppShell,
                                nsIToolkit *aToolkit,
                                nsWidgetInitData *aInitData)
@@ -429,7 +429,8 @@ nsresult nsCocoaWindow::CreateNativeWindow(const NSRect &aRect,
     [mWindow setHasShadow:YES];
   }
 
-  [mWindow setBackgroundColor:[NSColor whiteColor]];
+  [mWindow setBackgroundColor:[NSColor clearColor]];
+  [mWindow setOpaque:NO];
   [mWindow setContentMinSize:NSMakeSize(60, 60)];
   [mWindow disableCursorRects];
 
@@ -447,7 +448,7 @@ nsresult nsCocoaWindow::CreateNativeWindow(const NSRect &aRect,
 
 NS_IMETHODIMP nsCocoaWindow::CreatePopupContentView(const nsIntRect &aRect,
                              EVENT_CALLBACK aHandleEventFunction,
-                             nsIDeviceContext *aContext,
+                             nsDeviceContext *aContext,
                              nsIAppShell *aAppShell,
                              nsIToolkit *aToolkit)
 {
@@ -932,6 +933,15 @@ nsCocoaWindow::ConfigureChildren(const nsTArray<Configuration>& aConfigurations)
   return NS_OK;
 }
 
+LayerManager*
+nsCocoaWindow::GetLayerManager(bool *aAllowRetaining)
+{
+  if (mPopupContentView) {
+    return mPopupContentView->GetLayerManager(aAllowRetaining);
+  }
+  return nsnull;
+}
+
 LayerManager*
 nsCocoaWindow::GetLayerManager(LayerManagerPersistence, bool* aAllowRetaining)
 {
