@@ -45,12 +45,12 @@
 #include "nsCOMPtr.h"
 #include "nsIAtom.h"
 #include "nsILookAndFeel.h"
-#include "nsIDeviceContext.h"
 #include "nsNativeTheme.h"
-
 #include "gfxASurface.h"
 
 @class CellDrawView;
+@class NSProgressBarCell;
+class nsDeviceContext;
 
 class nsNativeThemeCocoa : private nsNativeTheme,
                            public nsITheme
@@ -59,30 +59,30 @@ public:
   nsNativeThemeCocoa();
   virtual ~nsNativeThemeCocoa();
 
-  NS_DECL_ISUPPORTS
+  NS_DECL_ISUPPORTS_INHERITED
 
   // The nsITheme interface.
-  NS_IMETHOD DrawWidgetBackground(nsIRenderingContext* aContext,
+  NS_IMETHOD DrawWidgetBackground(nsRenderingContext* aContext,
                                   nsIFrame* aFrame,
                                   PRUint8 aWidgetType,
                                   const nsRect& aRect,
                                   const nsRect& aDirtyRect);
-  NS_IMETHOD GetWidgetBorder(nsIDeviceContext* aContext, 
+  NS_IMETHOD GetWidgetBorder(nsDeviceContext* aContext, 
                              nsIFrame* aFrame,
                              PRUint8 aWidgetType,
-                             nsMargin* aResult);
+                             nsIntMargin* aResult);
 
-  virtual PRBool GetWidgetPadding(nsIDeviceContext* aContext,
+  virtual PRBool GetWidgetPadding(nsDeviceContext* aContext,
                                   nsIFrame* aFrame,
                                   PRUint8 aWidgetType,
-                                  nsMargin* aResult);
+                                  nsIntMargin* aResult);
 
-  virtual PRBool GetWidgetOverflow(nsIDeviceContext* aContext, nsIFrame* aFrame,
+  virtual PRBool GetWidgetOverflow(nsDeviceContext* aContext, nsIFrame* aFrame,
                                    PRUint8 aWidgetType, nsRect* aOverflowRect);
 
-  NS_IMETHOD GetMinimumWidgetSize(nsIRenderingContext* aContext, nsIFrame* aFrame,
+  NS_IMETHOD GetMinimumWidgetSize(nsRenderingContext* aContext, nsIFrame* aFrame,
                                   PRUint8 aWidgetType,
-                                  nsSize* aResult, PRBool* aIsOverridable);
+                                  nsIntSize* aResult, PRBool* aIsOverridable);
   NS_IMETHOD WidgetStateChanged(nsIFrame* aFrame, PRUint8 aWidgetType, 
                                 nsIAtom* aAttribute, PRBool* aShouldRepaint);
   NS_IMETHOD ThemeChanged();
@@ -90,57 +90,60 @@ public:
   PRBool WidgetIsContainer(PRUint8 aWidgetType);
   PRBool ThemeDrawsFocusForWidget(nsPresContext* aPresContext, nsIFrame* aFrame, PRUint8 aWidgetType);
   PRBool ThemeNeedsComboboxDropmarker();
-  virtual nsTransparencyMode GetWidgetTransparency(PRUint8 aWidgetType);
+  virtual Transparency GetWidgetTransparency(nsIFrame* aFrame, PRUint8 aWidgetType);
 
 protected:  
 
   nsresult GetSystemColor(PRUint8 aWidgetType, nsILookAndFeel::nsColorID& aColorID);
-  nsresult GetSystemFont(PRUint8 aWidgetType, nsSystemFontID& aFont);
   nsIntMargin RTLAwareMargin(const nsIntMargin& aMargin, nsIFrame* aFrame);
 
+  // Helpers for progressbar.
+  double GetProgressValue(nsIFrame* aFrame);
+  double GetProgressMaxValue(nsIFrame* aFrame);
+
   // HITheme drawing routines
   void DrawFrame(CGContextRef context, HIThemeFrameKind inKind,
-                 const HIRect& inBoxRect, PRBool inIsDisabled,
-                 PRInt32 inState);
+                 const HIRect& inBoxRect, PRBool inReadOnly,
+                 nsEventStates inState);
   void DrawProgress(CGContextRef context, const HIRect& inBoxRect,
                     PRBool inIsIndeterminate, PRBool inIsHorizontal,
-                    PRInt32 inValue, PRInt32 inMaxValue, nsIFrame* aFrame);
-  void DrawTab(CGContextRef context, HIRect inBoxRect, PRInt32 inState,
+                    double inValue, double inMaxValue, nsIFrame* aFrame);
+  void DrawTab(CGContextRef context, HIRect inBoxRect, nsEventStates inState,
                nsIFrame* aFrame);
   void DrawTabPanel(CGContextRef context, const HIRect& inBoxRect, nsIFrame* aFrame);
   void DrawScale(CGContextRef context, const HIRect& inBoxRect,
-                 PRBool inIsDisabled, PRInt32 inState,
-                 PRBool inDirection, PRBool inIsReverse,
-                 PRInt32 inCurrentValue,
-                 PRInt32 inMinValue, PRInt32 inMaxValue,
-                 nsIFrame* aFrame);
+                 nsEventStates inState, PRBool inDirection,
+                 PRBool inIsReverse, PRInt32 inCurrentValue, PRInt32 inMinValue,
+                 PRInt32 inMaxValue, nsIFrame* aFrame);
   void DrawCheckboxOrRadio(CGContextRef cgContext, PRBool inCheckbox,
                            const HIRect& inBoxRect, PRBool inSelected,
-                           PRBool inDisabled, PRInt32 inState, nsIFrame* aFrame);
-  void DrawSearchField(CGContextRef cgContext, const HIRect& inBoxRect, nsIFrame* aFrame);
-  void DrawPushButton(CGContextRef cgContext, const HIRect& inBoxRect, PRBool inIsDefault,
-                      PRBool inDisabled, PRInt32 inState, nsIFrame* aFrame);
+                           nsEventStates inState, nsIFrame* aFrame);
+  void DrawSearchField(CGContextRef cgContext, const HIRect& inBoxRect,
+                       nsIFrame* aFrame, nsEventStates inState);
+  void DrawPushButton(CGContextRef cgContext, const HIRect& inBoxRect,
+                      nsEventStates inState, nsIFrame* aFrame);
   void DrawButton(CGContextRef context, ThemeButtonKind inKind,
                   const HIRect& inBoxRect, PRBool inIsDefault, 
-                  PRBool inDisabled, ThemeButtonValue inValue,
-                  ThemeButtonAdornment inAdornment, PRInt32 inState, nsIFrame* aFrame);
-  void DrawDropdown(CGContextRef context, const HIRect& inBoxRect, PRInt32 inState,
-                    PRBool aIsEditable, nsIFrame* aFrame);
+                  ThemeButtonValue inValue, ThemeButtonAdornment inAdornment,
+                  nsEventStates inState, nsIFrame* aFrame);
+  void DrawDropdown(CGContextRef context, const HIRect& inBoxRect,
+                    nsEventStates inState, PRUint8 aWidgetType,
+                    nsIFrame* aFrame);
   void DrawSpinButtons(CGContextRef context, ThemeButtonKind inKind,
-                       const HIRect& inBoxRect,
-                       PRBool inDisabled, ThemeDrawState inDrawState,
-                       ThemeButtonAdornment inAdornment, PRInt32 inState,
+                       const HIRect& inBoxRect, ThemeDrawState inDrawState,
+                       ThemeButtonAdornment inAdornment, nsEventStates inState,
                        nsIFrame* aFrame);
   void DrawUnifiedToolbar(CGContextRef cgContext, const HIRect& inBoxRect,
-                          nsIFrame *aFrame);
+                          NSWindow* aWindow);
   void DrawStatusBar(CGContextRef cgContext, const HIRect& inBoxRect,
                      nsIFrame *aFrame);
+  void DrawResizer(CGContextRef cgContext, const HIRect& aRect, nsIFrame *aFrame);
 
   // Scrollbars
   void DrawScrollbar(CGContextRef aCGContext, const HIRect& aBoxRect, nsIFrame *aFrame);
-  void GetScrollbarPressStates (nsIFrame *aFrame, PRInt32 aButtonStates[]);
+  void GetScrollbarPressStates (nsIFrame *aFrame, nsEventStates aButtonStates[]);
   void GetScrollbarDrawInfo (HIThemeTrackDrawInfo& aTdi, nsIFrame *aFrame, 
-                             const HIRect& aRect, PRBool aShouldGetButtonStates);
+                             const CGSize& aSize, PRBool aShouldGetButtonStates);
   nsIFrame* GetParentScrollbarFrame(nsIFrame *aFrame);
 
 private:
@@ -150,6 +153,7 @@ private:
   NSSearchFieldCell* mSearchFieldCell;
   NSPopUpButtonCell* mDropdownCell;
   NSComboBoxCell* mComboBoxCell;
+  NSProgressBarCell* mProgressBarCell;
   CellDrawView* mCellDrawView;
 };
 
