@@ -41,13 +41,10 @@
 
 #include "GfxInfoWebGL.h"
 #include "GfxDriverInfo.h"
-#include "nsIPrefBranch2.h"
-#include "nsIPrefService.h"
 #include "nsCOMPtr.h"
 #include "nsCOMArray.h"
 #include "nsAutoPtr.h"
 #include "nsString.h"
-#include "nsServiceManagerUtils.h"
 #include "mozilla/Services.h"
 #include "nsIObserver.h"
 #include "nsIObserverService.h"
@@ -56,8 +53,9 @@
 #include "nsIDOM3Node.h"
 #include "nsIDOMNodeList.h"
 #include "nsTArray.h"
+#include "mozilla/Preferences.h"
 
-#if defined(MOZ_CRASHREPORTER) && defined(MOZ_ENABLE_LIBXUL)
+#if defined(MOZ_CRASHREPORTER)
 #include "nsExceptionHandler.h"
 #endif
 
@@ -89,7 +87,7 @@ StoreSpline(int ax, int ay, int bx, int by, int cx, int cy, int dx, int dy) {
 
 void
 CrashSpline(double tolerance, int ax, int ay, int bx, int by, int cx, int cy, int dx, int dy) {
-#if defined(MOZ_CRASHREPORTER) && defined(MOZ_ENABLE_LIBXUL)
+#if defined(MOZ_CRASHREPORTER)
   static bool annotated;
 
   if (!annotated) {
@@ -122,6 +120,7 @@ CrashSpline(double tolerance, int ax, int ay, int bx, int by, int cx, int cy, in
 
 
 using namespace mozilla::widget;
+using namespace mozilla;
 
 NS_IMPL_ISUPPORTS3(GfxInfoBase, nsIGfxInfo, nsIObserver, nsISupportsWeakReference)
 
@@ -171,16 +170,8 @@ GetPrefValueForFeature(PRInt32 aFeature, PRInt32& aValue)
   if (!prefname)
     return false;
 
-  nsCOMPtr<nsIPrefBranch2> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
-  if (prefs) {
-    PRInt32 val;
-    if (NS_SUCCEEDED(prefs->GetIntPref(prefname, &val))) {
-      aValue = val;
-      return true;
-    }
-  }
-
-  return false;
+  aValue = PR_FALSE;
+  return NS_SUCCEEDED(Preferences::GetInt(prefname, &aValue));
 }
 
 static void
@@ -190,10 +181,7 @@ SetPrefValueForFeature(PRInt32 aFeature, PRInt32 aValue)
   if (!prefname)
     return;
 
-  nsCOMPtr<nsIPrefBranch2> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
-  if (prefs) {
-    prefs->SetIntPref(prefname, aValue);
-  }
+  Preferences::SetInt(prefname, aValue);
 }
 
 static void
@@ -203,46 +191,25 @@ RemovePrefForFeature(PRInt32 aFeature)
   if (!prefname)
     return;
 
-  nsCOMPtr<nsIPrefBranch2> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
-  if (prefs) {
-    prefs->ClearUserPref(prefname);
-  }
+  Preferences::ClearUser(prefname);
 }
 
 static bool
-GetPrefValueForDriverVersion(nsACString& aVersion)
+GetPrefValueForDriverVersion(nsCString& aVersion)
 {
-  nsCOMPtr<nsIPrefBranch2> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
-  if (prefs) {
-    nsXPIDLCString version;
-    if (NS_SUCCEEDED(prefs->GetCharPref(SUGGESTED_VERSION_PREF,
-                                        getter_Copies(version)))) {
-      aVersion = version;
-      return true;
-    }
-  }
-
-  return false;
+  return NS_SUCCEEDED(Preferences::GetChar(SUGGESTED_VERSION_PREF, &aVersion));
 }
 
 static void
-SetPrefValueForDriverVersion(const nsAString& aVersion)
+SetPrefValueForDriverVersion(const nsString& aVersion)
 {
-  nsCOMPtr<nsIPrefBranch2> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
-  if (prefs) {
-    nsCAutoString ver = NS_LossyConvertUTF16toASCII(aVersion);
-    prefs->SetCharPref(SUGGESTED_VERSION_PREF,
-                       PromiseFlatCString(ver).get());
-  }
+  Preferences::SetChar(SUGGESTED_VERSION_PREF, aVersion);
 }
 
 static void
 RemovePrefForDriverVersion()
 {
-  nsCOMPtr<nsIPrefBranch2> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
-  if (prefs) {
-    prefs->ClearUserPref(SUGGESTED_VERSION_PREF);
-  }
+  Preferences::ClearUser(SUGGESTED_VERSION_PREF);
 }
 
 // <foo>Hello</foo> - "Hello" is stored as a child text node of the foo node.
@@ -677,7 +644,7 @@ GfxInfoBase::LogFailure(const nsACString &failure)
     mFailures[mFailureCount++] = failure;
 
     /* record it in the crash notes too */
-#if defined(MOZ_CRASHREPORTER) && defined(MOZ_ENABLE_LIBXUL)
+#if defined(MOZ_CRASHREPORTER)
     CrashReporter::AppendAppNotesToCrashReport(failure);
 #endif
   }
