@@ -43,7 +43,7 @@
 #include "nsIFrame.h"
 #include "nsIPresShell.h"
 #include "nsPresContext.h"
-#include "nsIEventStateManager.h"
+#include "nsEventStateManager.h"
 #include "nsString.h"
 #include "nsINameSpaceManager.h"
 #include "nsIDOMHTMLInputElement.h"
@@ -51,6 +51,7 @@
 #include "nsThemeConstants.h"
 #include "nsIComponentManager.h"
 #include "nsPIDOMWindow.h"
+#include "nsProgressFrame.h"
 
 nsNativeTheme::nsNativeTheme()
 : mAnimatedContentTimeout(PR_UINT32_MAX)
@@ -91,7 +92,7 @@ nsNativeTheme::GetContentState(nsIFrame* aFrame, PRUint8 aWidgetType)
   if (!shell)
     return nsEventStates();
 
-  nsIEventStateManager* esm = shell->GetPresContext()->EventStateManager();
+  nsEventStateManager* esm = shell->GetPresContext()->EventStateManager();
   nsEventStates flags = esm->GetContentState(aFrame->GetContent(), PR_TRUE);
   
   if (isXULCheckboxRadio && aWidgetType == NS_THEME_RADIO) {
@@ -251,6 +252,19 @@ nsNativeTheme::IsWidgetStyled(nsPresContext* aPresContext, nsIFrame* aFrame,
     }
   }
 
+  /**
+   * Progress bar appearance should be the same for the bar and the container
+   * frame. nsProgressFrame owns the logic and will tell us what we should do.
+   */
+  if (aWidgetType == NS_THEME_PROGRESSBAR_CHUNK ||
+      aWidgetType == NS_THEME_PROGRESSBAR) {
+    nsProgressFrame* progressFrame = do_QueryFrame(aWidgetType == NS_THEME_PROGRESSBAR_CHUNK
+                                       ? aFrame->GetParent() : aFrame);
+    if (progressFrame) {
+      return !progressFrame->ShouldUseNativeStyle();
+    }
+  }
+
   return (aWidgetType == NS_THEME_BUTTON ||
           aWidgetType == NS_THEME_TEXTFIELD ||
           aWidgetType == NS_THEME_TEXTFIELD_MULTILINE ||
@@ -443,16 +457,28 @@ nsNativeTheme::IsNextToSelectedTab(nsIFrame* aFrame, PRInt32 aOffset)
 
 // progressbar:
 PRBool
-nsNativeTheme::IsIndeterminateProgress(nsIFrame* aFrame)
+nsNativeTheme::IsIndeterminateProgress(nsIFrame* aFrame,
+                                       nsEventStates aEventStates)
 {
   if (!aFrame)
     return PR_FALSE;
 
+  if (aFrame->GetContent()->IsHTML(nsWidgetAtoms::progress)) {
+    return aEventStates.HasState(NS_EVENT_STATE_INDETERMINATE);
+  }
+
   return aFrame->GetContent()->AttrValueIs(kNameSpaceID_None, nsWidgetAtoms::mode,
                                            NS_LITERAL_STRING("undetermined"),
                                            eCaseMatters);
 }
 
+PRBool
+nsNativeTheme::IsVerticalProgress(nsIFrame* aFrame)
+{
+  return aFrame &&
+         aFrame->GetStyleDisplay()->mOrient == NS_STYLE_ORIENT_VERTICAL;
+}
+
 // menupopup:
 PRBool
 nsNativeTheme::IsSubmenu(nsIFrame* aFrame, PRBool* aLeftOfParent)
@@ -531,11 +557,7 @@ nsNativeTheme::Notify(nsITimer* aTimer)
   for (PRUint32 index = 0; index < count; index++) {
     nsIFrame* frame = mAnimatedContentList[index]->GetPrimaryFrame();
     if (frame) {
-#ifdef MOZ_ENABLE_LIBXUL
       frame->InvalidateOverflowRect();
-#else
-      frame->InvalidateOverflowRectExternal();
-#endif
     }
   }
 
